global data "C:\Users\hoaisonkt\Dropbox\Dang xu ly\Heatwave\shpgsomatch\Example"
global shpfile "C:\Users\hoaisonkt\Dropbox\Dang xu ly\Heatwave\shpgsomatch\GSO code for shp 2018\Dta file"

cd "$data"

* Step 1: Get household income from Ho3 - VHLSS 2014
tempfile hh_inc
use Ho3, clear
  keep tinh huyen xa diaban hoso thubq
  rename thubq inc_month
  lab var inc_month "Average monthly income per capita"
save `hh_inc', replace

* Step 2: Calculate cdd25 for each station in Jun 2014 
tempfile cdd25
import delimited "1041206.csv", clear
  replace station=substr(station,length(station)-4, 5)
  gen date_temp=date( date, "YMD" )
     format %td date_temp
  gen month = month(date_temp)
  gen year = year(date_temp)
 
  replace tavg= (tavg-32)*5/9 //convert F to C

  gen cdd25_day = tavg-25
     replace cdd25_day=0 if cdd25_day<=0
  
  bysort station year month: egen cdd25=total(cdd25_day)
  keep if month==6 & year==2014  
  keep station name latitude longitude cdd25
  duplicates drop
save `cdd25', replace

* Step 3: assign cdd25 of the nearest station to each ward
tempfile ward_cdd25
use `cdd25', clear
  cross using "$shpfile\vndb_centroids"
  
  * Calculate the distance from each station to each ward
  gen dlon=(_pi/180)*( x_center- longitude)
  gen dlat=(_pi/180)*( y_center- latitude)
   
  gen a = (sin(dlat/2))^2 + cos( latitude ) * cos( y_center ) * (sin(dlon/2))^2
  gen c = 2 * atan2( sqrt(a), sqrt(1-a) )
  gen d= 6373*c
   
  * Determine and keep the nearest station
  bysort id: egen nearest_station=min(d)
  keep if nearest_station==d

  keep station name cdd25 id city_code city district_code district ward_code ward wardtype 
  codebook city_code //missing "":  9/11,163 - can delete
  drop if city_code==""

save `ward_cdd25', replace

* Step 4: merge cdd25 to hh_inc and calculate correlation 
use `ward_cdd25', clear
  * rename city district and ward code to match with VHLSS
  rename city_code tinh
  rename district_code huyen
  rename ward_code xa
  destring tinh huyen xa, replace
  
  merge 1:m tinh huyen xa using `hh_inc'
  /*Result                           # of obs.
    -----------------------------------------
    not matched                         8,106
        from master                     8,043  (_merge==1)
        from using                         63  (_merge==2)

    matched                             9,336  (_merge==3)
    ----------------------------------------- */
  ** unmatch 8,106 cases because data on cdd25 is for all wards in Vietnam. Meanwhile
  ** data on VHLSS is only for a sample of wards in Vietnam. Unmatch from using (_merge==2)
  ** is only 63 meaning that there are 63 household can't determine cdd25. The reasons may come
  ** from the fact that the VHLSS data is on 2014 while shpfile is for 2018, thus, some wards
  ** in VHLSS are no longer exists in shpfile
  
  keep if _merge==3
  drop _merge

  pwcorr cdd25 inc_month, star(0.05) sig
/*           |    cdd25 inc_mo~h
-------------+------------------
       cdd25 |   1.0000 
   inc_month |   0.0020   1.0000 
             |   0.8489*/

  bysort id: egen hhinc_av=mean(inc_month)
     label var hhinc_av "Average monthly income per cap of each ward"
  keep id city hhinc_av cdd25
  duplicates drop
  
  * Maps hh income for 3133 wards across VN - Orange color
  format hhinc_av %6.0f
     spmap hhinc_av if cdd25!=. & hhinc_av!=. using "$shpfile\vncoord_centroids", id(id) fcolor(Oranges) ocolor(none ..) legtitle("Income per capita, 2014")
  graph export hhinc_av.png, replace  

  
  
