/*--------------------------------------------------------------------------------------------------------*/
/*--- Stata ver 14                                                                                 ------ */
/*--- This program is to assign official GSO admin code to GADM shapefile by creating a map table  ------ */
/*--- that maps 1:1 city name, district name, ward name, ward type of GSO list and the shapefile   ------ */
/*--------------------------------------------------------------------------------------------------------*/


* Set up global for path
global shp_src "C:\Users\hoaisonkt\OneDrive - National Economics University\Data bank\gadm36_VNM_shp"
global gso_src "C:\Users\hoaisonkt\OneDrive - National Economics University\Data bank\Don vi hanh chinh"
global results "C:\Users\hoaisonkt\Dropbox\Dang xu ly\Heatwave\shpgsomatch\GSO code for shp 2018\Dta file"
global excel_des "C:\Users\hoaisonkt\Dropbox\Dang xu ly\Heatwave\shpgsomatch\GSO code for Shp 2018\Excel file" 

cd "$results"

/*--------------------------------------------------------------------------------------------------------*/
/*-------------------------------- Step 1: Normalization - semantic --------------------------------------*/
/*--------------------------------------------------------------------------------------------------------*/

** GSO list **
clear
   import excel "$gso_src\xa20141231.xls", sheet("Sheet1") firstrow

   rename Tên ward
   rename Cấp wardtype
   rename QuậnHuyện district
   rename MãQH district_code
   rename Mã ward_code
   rename TỉnhThànhPhố city
   rename MãTP city_code

   * Gen normalized fields
   gen ward_norm = ward
   gen wardtype_norm = wardtype
   gen district_norm = district
   gen city_norm = city
   
   * Separate ward type from ward name */
   replace ward_norm=subinstr(ward_norm, "Thị Trấn" , "Thị trấn", 1) //change Thị Trấn -> Thị trấn
   * (12 real changes made)
   replace ward_norm = subinstr(ward_norm, wardtype_norm, "",1) //separate wardtype from ward
   * (11,161 real changes made) - = 11,161 obs
   replace ward_norm=trim(ward_norm) // remove leading and trait spaces

   /* Separate district type from district name */
   replace district_norm = subinstr(district_norm, "Quận", "", 1)
   * (586 real changes made)
   replace district_norm = subinstr(district_norm, "Huyện", "", 1)
   * (8,926 real changes made)
   replace district_norm = subinstr(district_norm, "Thị xã", "", 1)
   * (572 real changes made)
   replace district_norm = subinstr(district_norm, "Thị Xã", "", 1)
   * (15 real changes made)
   replace district_norm = subinstr(district_norm, "Thành phố", "", 1)
   * (1,038 real changes made)
   replace district_norm = subinstr(district_norm, "Thành Phố", "", 1)
   * (24 real changes made)
   dis 586+8926+572+15+1038+24
   * 11161 = 11161 số obs
   replace district_norm = trim(district_norm)

   /* Separate city type from city name */
   replace city_norm =subinstr(city_norm, "Thành phố", "", 1)
   * (1,270 real changes made)
   replace city_norm = subinstr(city_norm, "Tỉnh", "", 1)
   * (9,891 real changes made)
   dis 1270+9891
   * 11161 = 11161 số obs
   replace city_norm=trim(city_norm)

save gso.dta, replace

** Shapefile **
shp2dta using "$shp_src\gadm36_VNM_3", database(vndb_centroids) coordinates(vncoord_centroids) genid(id) gencentroids(center) replace

use vndb_centroids, clear
/* Change variable name */
   rename NAME_0 country
      label variable country "Country name"
   rename NAME_1 city
      label variable city "City name"
   rename NAME_2 district
      label variable district "District name"     
   rename NAME_3 ward
      label variable ward "Ward name"  
   rename TYPE_3 wardtype
      label variable wardtype "Ward type"

   * Gen normalized fields
   gen ward_norm = ward
   gen wardtype_norm = wardtype
   gen district_norm = district
   gen city_norm = city
      
   replace district_norm = subinstr(district_norm, "Thành Phố", "", 1) //Thành Phố Bắc Kan
   * (24 real changes made)
   replace district_norm = subinstr(district_norm, "Quận", "", 1) //Quận 10 - (Hồ Chí Minh)	
   * (160 real changes made)
   replace district_norm = subinstr(district_norm, "Thị Xã", "",1) //Thị Xã Buôn Hồ, Thị Xã Mường Lay
   * (15 real changes made)
   replace district_norm = regexr(district_norm, "\((.)+\)", "")
   * (66 real changes made)

   replace ward_norm = subinstr(ward_norm, "Phường", "", 1) //Phường 10 - (Hồ Chí Minh)
   * (314 real changes made)
   replace ward_norm =   ustrregexrf(ward_norm, "\((.)+\)", "")
   * (57 real changes made)
   
   replace district_norm=trim(district_norm) //remove leading and trail blank
   replace ward_norm=trim(ward_norm)   
save vndb_centroids, replace


/*--------------------------------------------------------------------------------------------------------*/
/*----------- Step 2: Gen map table for matched case after normalization ---------------------------------*/
/*------------------- and store unmatched case in 2 different file: gso_temp and shp_temp ----------------*/
/*--------------------------------------------------------------------------------------------------------*/

use vndb_centroids, clear
  merge 1:1 city_norm district_norm ward_norm wardtype_norm using gso
  /*Result                           # of obs.
    -----------------------------------------
    not matched                           418
        from master                       210  (_merge==1)
        from using                        208  (_merge==2)

    matched                            10,953  (_merge==3)
    ----------------------------------------- */
   
  * Form a mapping table - store matching key first
  preserve
    keep if _merge==3
	keep city_norm district_norm ward_norm wardtype_norm
	rename * shp_*
	local varlist "city_norm district_norm ward_norm wardtype_norm"
	foreach v of local varlist {
	   gen gso_`v'=shp_`v'
	}
	
	gen type = 1 // 1- matched after normalization
	gen comment=""
    save map_table, replace
  restore
  
  * Saving unmatch cases to separate files
  tempfile shp_temp gso_temp
  preserve
    keep if _merge==1 // from shp
      drop _merge
	  replace ward_norm=subinstr(ward_norm, "N.T", "Nông Trường",1) 
      replace ward_norm=subinstr(ward_norm, "NT", "Nông Trường",1) 
    save `shp_temp', replace 
  restore
  
  preserve
    keep if _merge==2 // from gso
      drop _merge
	  replace ward_norm=subinstr(ward_norm, "Thị trấn N.T", "Thị trấn Nông Trường",1) //Thị trấn N.T ->"Thị trấn Nông Trường"
      * (1 real change made)
      replace ward_norm=subinstr(ward_norm, "Thị trấn NT", "Thị trấn Nông Trường",1) //"Thị trấn NT" - > "Thị trấn Nông Trường"    
	save `gso_temp', replace 
  restore  
  

/*--------------------------------------------------------------------------------------------------------*/
/*----------- Step 3: Remove leading zero in names of GSO list -------------------------------------------*/
/*------------------- Merge again, store matched cases in map table --------------------------------------*/
/*------------------- and export the unmatched case to excel to match manually ---------------------------*/
/*--------------------------------------------------------------------------------------------------------*/
  
  * Removing leading zero in names
  use `gso_temp', clear
    * Gen new var to store leading zero
	gen gso_ward_norm = ward_norm
	replace ward_norm = substr(ward_norm,2, length(ward_norm)-1) if substr(ward_norm,1,1)=="0"
  save `gso_temp', replace
  
  * Merge again
  use `gso_temp', clear
    merge 1:1 city_norm district_norm ward_norm wardtype_norm using `shp_temp'
    /*Result                           # of obs.
    -----------------------------------------
    not matched                           238
        from master                       118  (_merge==1)
        from using                        120  (_merge==2)

    matched                                90  (_merge==3)
    ----------------------------------------- */
    
	* Save matched cases to merge_table
	preserve
   	  keep if _merge==3
      keep *_norm
      
	  gen gso_wardtype_norm = wardtype_norm
	  gen gso_district_norm = district_norm
	  gen gso_city_norm = city_norm
	  
	  rename ward_norm shp_ward_norm
	  rename wardtype_norm shp_wardtype_norm
	  rename district_norm shp_district_norm
	  rename city_norm shp_city_norm
	  
	  gen type=2
      gen comment=""
	  
      order shp_ward_norm shp_wardtype_norm shp_district_norm shp_city_norm ///
          gso_city_norm gso_district_norm gso_ward_norm gso_wardtype_norm ///
		  type comment
      
	  append using map_table
	  save map_table, replace
	restore
	
	* Export the rest to excel to merge manually
	drop gso_ward_norm
	keep if _merge!=3
	keep *_norm _merge
	export excel using "$excel_des\List of matching manual cases.xlsx", sheet("Raw list", replace)
	  

/*--------------------------------------------------------------------------------------------------------*/
/*----------- Step 4: Import excel file storing manually matching cases ----------------------------------*/
/*------------------- Append the new data from excel to map table  ---------------------------------------*/
/*--------------------------------------------------------------------------------------------------------*/
	  
* Matching manually in excel file in sheet "Matching Manually" then import back to stata
clear
   	import excel using "$excel_des\List of matching manual cases.xlsx", sheet("Matching manually") firstrow
    destring type, replace
	
	append using map_table
    lab var type "Type of mismatch"
    lab def Type 1 "Atomic normalization" 2 "Leading zero in names" 3 "Capitalization" ///
	             4 "Tone mark position" 5 "Ward changes" 6 "District changes" 7 "Others" 8 "Unmatch"
    lab val type Type
	
	/* This part is to generate some unrealistic value for city_norm for both missing gso and shp cases in map table. 
	* There are some wards in gso that donot exist in shapefile and vice versa. These wards have empty value          
	* so Stata treated as duplicated cases which cause error in merging with GSO file or Shp file latter. 
	* We assign city_norm for the empty case = 100000 + row number so these cases are not duplicated any more. */ 
	gen missing_norm = 100000 + _n
	tostring missing_norm, replace
	replace gso_city_norm = missing_norm if gso_city_norm==""
	replace shp_city_norm = missing_norm if shp_city_norm==""
	drop missing_norm
save map_table, replace
  

/*--------------------------------------------------------------------------------------------------------*/
/*----------- Step 5: Use the map table to merge shp and gso ---------------------------------------------*/
/*------------------- Firt, match shp file with maptable -------------------------------------------------*/
/*------------------- Second, match gso with new file ----------------------------------------------------*/ 
/*--------------------------------------------------------------------------------------------------------*/

tempfile final
use vndb_centroids, clear
  rename * shp_*
  merge 1:1  shp_ward_norm shp_wardtype_norm shp_district_norm shp_city_norm using map_table
  /*Result                           # of obs.
    -----------------------------------------
    not matched                             7
        from master                         0  (_merge==1)
        from using                          7  (_merge==2)

    matched                            11,163  (_merge==3)
    ----------------------------------------- */
  keep if _merge==3 // _merge==1 (shapefile) = 0 - can drop
  drop _merge 
save `final', replace  

use gso, clear 
  rename * gso_*
  merge 1:1 gso_ward_norm gso_wardtype_norm gso_district_norm gso_city_norm using `final'
  /*Result                           # of obs.
    -----------------------------------------
    not matched                            16
        from master                         7  (_merge==1)
        from using                          9  (_merge==2)

    matched                            11,154  (_merge==3)
    -----------------------------------------
 */
  
 keep if _merge!=1 // drop unmatch obs from gso list
 * (7 observations deleted)
 keep *_code shp_*
 rename gso_* *
 rename shp_* *
 order id x_center y_center ward_code district_code city_code ward wardtype district city country
 sort id
save vndb_centroids, replace
